==> collaboration-tools-241.11761.10-EAP-SNAPSHOT-sources.jar: com/intellij/collaboration/ui/codereview/list/error/ErrorStatusPanelFactory.kt <==
// Copyright 2000-2022 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package com.intellij.collaboration.ui.codereview.list.error

import com.intellij.collaboration.ui.util.name
import com.intellij.ide.BrowserUtil
import com.intellij.openapi.util.text.HtmlBuilder
import com.intellij.openapi.util.text.HtmlChunk
import com.intellij.ui.HyperlinkAdapter
import com.intellij.util.ui.HTMLEditorKitBuilder
import com.intellij.util.ui.NamedColorUtil
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.launch
import org.jetbrains.annotations.Nls
import java.awt.event.ActionEvent
import java.awt.event.ActionListener
import java.awt.event.KeyEvent
import javax.swing.Action
import javax.swing.JComponent
import javax.swing.JEditorPane
import javax.swing.KeyStroke
import javax.swing.event.HyperlinkEvent

object ErrorStatusPanelFactory {
  private const val ERROR_ACTION_HREF = "ERROR_ACTION"

  @JvmOverloads
  fun <T> create(
    scope: CoroutineScope,
    errorState: Flow<T?>,
    errorPresenter: ErrorStatusPresenter<T>,
    alignment: Alignment = Alignment.CENTER
  ): JComponent {
    val htmlEditorPane = JEditorPane().apply {
      editorKit = HTMLEditorKitBuilder().withWordWrapViewFactory().build()
      foreground = NamedColorUtil.getErrorForeground()
      isFocusable = true
      isEditable = false
      isOpaque = false
    }

    Controller(scope, errorState, errorPresenter, htmlEditorPane, alignment)

    return htmlEditorPane
  }

  private class Controller<T>(
    scope: CoroutineScope,
    errorState: Flow<T?>,
    private val errorPresenter: ErrorStatusPresenter<T>,
    private val htmlEditorPane: JEditorPane,
    private val alignment: Alignment
  ) {
    private var action: Action? = null

    init {
      htmlEditorPane.apply {
        addHyperlinkListener(object : HyperlinkAdapter() {
          override fun hyperlinkActivated(event: HyperlinkEvent) {
            if (event.description == ERROR_ACTION_HREF) {
              val actionEvent = ActionEvent(htmlEditorPane, ActionEvent.ACTION_PERFORMED, "perform")
              action?.actionPerformed(actionEvent)
            }
            else {
              BrowserUtil.browse(event.description)
            }
          }
        })
        registerKeyboardAction(
          ActionListener { action?.actionPerformed(it) },
          KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
          JComponent.WHEN_FOCUSED
        )
      }

      scope.launch {
        errorState.collect { error ->
          update(error)
        }
      }
    }

    private fun update(error: T?) {
      if (error == null) {
        htmlEditorPane.text = ""
        htmlEditorPane.isVisible = false
        return
      }

      val errorTextBuilder = HtmlBuilder().apply {
        appendP(errorPresenter.getErrorTitle(error))
        val errorTitle = errorPresenter.getErrorDescription(error)
        if (errorTitle != null) {
          appendP(errorTitle)
        }
      }

      val errorAction = errorPresenter.getErrorAction(error)
      if (errorAction != null) {
        action = errorAction
        errorTextBuilder.appendP(HtmlChunk.link(ERROR_ACTION_HREF, errorAction.name.orEmpty()))
      }

      htmlEditorPane.text = errorTextBuilder.wrapWithHtmlBody().toString()
      htmlEditorPane.isVisible = true
    }

    private fun HtmlBuilder.appendP(chunk: HtmlChunk): HtmlBuilder = append(HtmlChunk.p().attr("align", alignment.htmlValue).child(chunk))

    private fun HtmlBuilder.appendP(@Nls text: String): HtmlBuilder = appendP(HtmlChunk.text(text))
  }

  enum class Alignment(val htmlValue: String) {
    LEFT("left"),
    CENTER("center");
  }
}

==> collaboration-tools-241.17011.2-EAP-SNAPSHOT-sources.jar: com/intellij/collaboration/ui/codereview/list/error/ErrorStatusPanelFactory.kt <==
// Copyright 2000-2022 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package com.intellij.collaboration.ui.codereview.list.error

import com.intellij.collaboration.ui.util.name
import com.intellij.ide.BrowserUtil
import com.intellij.openapi.util.text.HtmlBuilder
import com.intellij.openapi.util.text.HtmlChunk
import com.intellij.ui.HyperlinkAdapter
import com.intellij.util.ui.HTMLEditorKitBuilder
import com.intellij.util.ui.NamedColorUtil
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.launch
import org.jetbrains.annotations.Nls
import java.awt.event.ActionEvent
import java.awt.event.ActionListener
import java.awt.event.KeyEvent
import javax.swing.Action
import javax.swing.JComponent
import javax.swing.JEditorPane
import javax.swing.KeyStroke
import javax.swing.event.HyperlinkEvent

object ErrorStatusPanelFactory {

  @JvmOverloads
  fun <T> create(
    scope: CoroutineScope,
    errorState: Flow<T?>,
    errorPresenter: ErrorStatusPresenter<T>,
    alignment: Alignment = Alignment.CENTER
  ): JComponent {
    val htmlEditorPane = JEditorPane().apply {
      editorKit = HTMLEditorKitBuilder().withWordWrapViewFactory().build()
      foreground = NamedColorUtil.getErrorForeground()
      isFocusable = true
      isEditable = false
      isOpaque = false
    }

    Controller(scope, errorState, errorPresenter, htmlEditorPane, alignment)

    return htmlEditorPane
  }

  private class Controller<T>(
    scope: CoroutineScope,
    errorState: Flow<T?>,
    private val errorPresenter: ErrorStatusPresenter<T>,
    private val htmlEditorPane: JEditorPane,
    private val alignment: Alignment
  ) {
    private var action: Action? = null

    init {
      htmlEditorPane.apply {
        addHyperlinkListener(object : HyperlinkAdapter() {
          override fun hyperlinkActivated(event: HyperlinkEvent) {
            if (event.description == ErrorStatusPresenter.ERROR_ACTION_HREF) {
              val actionEvent = ActionEvent(htmlEditorPane, ActionEvent.ACTION_PERFORMED, "perform")
              action?.actionPerformed(actionEvent)
            }
            else {
              BrowserUtil.browse(event.description)
            }
          }
        })
        registerKeyboardAction(
          ActionListener { action?.actionPerformed(it) },
          KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
          JComponent.WHEN_FOCUSED
        )
      }

      scope.launch {
        errorState.collect { error ->
          update(error)
        }
      }
    }

    private fun update(error: T?) {
      if (error == null) {
        htmlEditorPane.text = ""
        htmlEditorPane.isVisible = false
        return
      }

      when (errorPresenter) {
        is ErrorStatusPresenter.HTML -> updateHTMLPresenter(error, errorPresenter)
        is ErrorStatusPresenter.Text -> updateTextPresenter(error, errorPresenter)
      }
      htmlEditorPane.isVisible = true
    }

    private fun <T> updateHTMLPresenter(error: T, errorPresenter: ErrorStatusPresenter.HTML<T>) {
      val errorAction = errorPresenter.getErrorAction(error)
      action = errorAction

      val htmlBody = errorPresenter.getHTMLBody(error)
      htmlEditorPane.text = htmlBody
    }

    private fun <T> updateTextPresenter(error: T, errorPresenter: ErrorStatusPresenter.Text<T>) {
      val errorTextBuilder = HtmlBuilder().apply {
        appendP(errorPresenter.getErrorTitle(error))
        val errorTitle = errorPresenter.getErrorDescription(error)
        if (errorTitle != null) {
          appendP(errorTitle)
        }
      }

      val errorAction = errorPresenter.getErrorAction(error)
      action = errorAction
      if (errorAction != null) {
        errorTextBuilder.appendP(HtmlChunk.link(ErrorStatusPresenter.ERROR_ACTION_HREF, errorAction.name.orEmpty()))
      }

      htmlEditorPane.text = errorTextBuilder.wrapWithHtmlBody().toString()
    }

    private fun HtmlBuilder.appendP(chunk: HtmlChunk): HtmlBuilder = append(HtmlChunk.p().attr("align", alignment.htmlValue).child(chunk))

    private fun HtmlBuilder.appendP(@Nls text: String): HtmlBuilder = appendP(HtmlChunk.text(text))
  }

  enum class Alignment(val htmlValue: String) {
    LEFT("left"),
    CENTER("center");
  }
}
